var Triangle = {
    length: 50,
    maxAngle: 90,
    currentAngle: 45,
    xBase: 0,
    yBase: 0,
    left: 5,
    top: 5,
    width: 0,
    draggable: false,
    dragFunction: null,
    hasHandle: false,
    drawLegs: false,
    drawArc: false,
    drawShadow: false,
    showAngle: false,
    canvas: null,
    sineMsg: "",
    cosineMsg: "",
    ctx: null,
    
    init: function(canvasId, settings) {
        Object.assign(this, settings);
        this.canvas = document.getElementById(canvasId);
        this.ctx = this.canvas.getContext("2d");
        this.xBase = this.left + this.length;
        this.yBase = this.top + this.length + ((this.showAngle) ? 40 : 0);

        if (this.draggable) {
           this.canvas.onmousedown = this.downFunction.bind(this);
           
           this.canvas.onmouseup = function(evt) {
               evt.target.onmousemove = null;
           }
        }
    },

    cartesian: function(degrees) {
        var radians = degrees * Math.PI / 180;
        x = this.xBase + (this.length * Math.cos(radians));
        y = this.yBase - (this.length * Math.sin(radians));
        return([x,y]);
    },
    

    draw: function() {
        var [x,y] = this.cartesian(this.currentAngle);
        var ctx = this.ctx;
        var yShadow = this.yBase + 7;
        var xShadow = this.xBase + this.length + 7;
        var handleExtra = (this.hasHandle) ? 4 : 1;
        var shadowExtra = (this.drawShadow) ? 7 : 0
        
        ctx.fillStyle = "#fff";
        ctx.fillRect(this.left - handleExtra, this.yBase - this.length - handleExtra,
                     2 * (this.length + handleExtra) + shadowExtra,
                     2 * (this.length + handleExtra) + shadowExtra);
        if (this.showAngle) {
            ctx.fillRect(this.left, this.top, this.left + 2.5 * this.length, 40);
        }
        ctx.strokeStyle = (this.draggable ? "black" : "#00c000");
        ctx.lineWidth = 2;
        drawLine(ctx, this.xBase, this.yBase, x, y);

        if (this.drawArc) {
            ctx.strokeStyle = "#ccc";
            ctx.lineWidth = 1;
            ctx.beginPath();
            ctx.arc(this.xBase, this.yBase, this.length, 0, -(this.maxAngle * Math.PI / 180), true);
            ctx.stroke();
        }
        
        if (this.drawLegs) {
            // height of triangle
            ctx.strokeStyle = "blue";
            drawLine(ctx, x, y, x, this.yBase);
            ctx.strokeStyle = "red";
            drawLine(ctx, this.xBase, this.yBase, x, this.yBase);
        }
        
        if (this.hasHandle) {
            ctx.fillStyle = "#080";
            ctx.beginPath();
            ctx.arc(x, y, 4, 0, 2 * Math.PI);
            ctx.fill();
        }
        
        if (this.drawShadow) {
            ctx.strokeStyle = "#888";
            ctx.lineWidth = 3;
            drawLine(ctx, this.xBase, yShadow, x, yShadow); // bottom
            drawLine(ctx, xShadow, this.yBase, xShadow, y); // right
        }
        
        if (this.showAngle) {
            var msg = "Angle: " + Math.floor(this.currentAngle) + "\u00b0";
            ctx.fillStyle = "black";
            ctx.font = "14px sans-serif";
            ctx.fillText(msg, this.left + 5, this.top + 12);
        }
        
        if (this.showRatios) {
            var cosMsg = this.cosineMsg + ": " +  Math.cos(this.currentAngle * Math.PI / 180).toFixed(3);
            var sinMsg = this.sineMsg + ": " + Math.sin(this.currentAngle * Math.PI / 180).toFixed(3);
            /*
            var cosXPos = (this.xBase + x - ctx.measureText(cosMsg).width) / 2.0;
            var sinXPos = (this.currentAngle <= 90 || this.currentAngle >= 270) ?
                x + 5 : x - (ctx.measureText(sinMsg).width + 5);
            var sinYPos = (this.yBase + y) / 2.0;
            */
            ctx.font = "14px sans-serif";
            ctx.fillStyle = "red";
            ctx.fillText(cosMsg, this.left + 5, this.top + 26);
            ctx.fillStyle = "blue";
            ctx.fillText(sinMsg, this.left + 5, this.top + 40);
        }
    },
    
    downFunction: function(evt) {
        evt.target.onmousemove = this.dragFunction.bind(this);
        this.dragFunction(evt);
    },

    dragFunction: function(evt) {
        var x = evt.offsetX;
        var y = evt.offsetY;
        
        var xDiff = (x - this.xBase);
        var yDiff = (this.yBase - y);
        var hypotenuse = Math.sqrt(xDiff * xDiff + yDiff * yDiff);

        if (hypotenuse >= 3) {
            testAngle = Math.floor(Math.asin(yDiff / hypotenuse) * 180 / Math.PI);
            if (xDiff > 0) {
                if (testAngle < 0) {
                    testAngle = 360 + testAngle;
                }
            } else {
                testAngle = 180 - testAngle;
            }

            if (testAngle <= this.maxAngle) {
                this.currentAngle = testAngle;
                this.draw();
            }
        }
    }
}
